@@ -1,12 +1,14 @@
 from __future__ import annotations
 
+import filecmp
 import os
-import re
 import shutil
+import subprocess
 import tempfile
 import zipfile
 from pathlib import Path
 
+import pathspec
 import pytest
 import toml
 from rich.panel import Panel
@@ -56,13 +58,16 @@ class Exercise:
 
     def zip(self, to_tmp_dir: bool = False, verbose: bool = False) -> Path:
         console.info('Compressing exercise contents', cr=verbose)
+        exclude_patterns = pathspec.PathSpec.from_lines(
+            'gitwildmatch', self.config.get('exclude_from_zip', [])
+        )
         zip_path = tempfile.mkstemp(suffix='.zip')[1] if to_tmp_dir else self.zipname
         zip_file = Path(zip_path)
         with zipfile.ZipFile(zip_file, 'w') as archive:
             for file in self.files:
                 if file.name == self.zipname:
                     continue
-                if re.search(settings.ZIP_IGNORED_PATTERNS_RE, str(file)):
+                if exclude_patterns.match_file(str(file)):
                     if verbose:
                         console.warning(f'Ignoring {file}')
                     continue
@@ -91,10 +96,18 @@ class Exercise:
         os.system(f'{sysutils.get_open_cmd()} docs/README.pdf')
 
     def update(self, src_dir: Path, backup: bool = True):
-        for file in src_dir.glob('**/*'):
-            if file.is_file():
+        backup_files = pathspec.PathSpec.from_lines(
+            'gitwildmatch', self.config.get('backup_on_update', [])
+        )
+        for dirpath, dirs, files in os.walk(src_dir):
+            for filename in files:
+                file = Path(dirpath) / filename
                 rel_file = file.relative_to(src_dir)
-                if backup and rel_file.exists() and str(rel_file) in self.config.get('todo', []):
+                if (
+                    backup
+                    and backup_files.match_file(str(rel_file))
+                    and not filecmp.cmp(rel_file, file, shallow=False)
+                ):
                     backup_file = rel_file.with_suffix(rel_file.suffix + '.bak')
                     console.debug(f'Backup {rel_file} → {backup_file}')
                     shutil.copy(rel_file, backup_file)
@@ -116,14 +129,13 @@ class Exercise:
                 console.error(monad.payload)
             zipfile.unlink(missing_ok=True)
 
-    def test(self):
-        if Path('requirements.txt').exists():
-            console.print("""[red]Attention[/red]
-A [i]requirements.txt[/i] file is detected in the current directory!
-pypas is not able to directly test this exercise.
-Please [i]install dependencies[/i] (usually inside a virtualenv) and run: [note]pytest""")
+    def test(self, args: list[str]):
+        if test_cmd := self.config.get('test_cmd'):
+            test_cmd = f'{test_cmd} {" ".join(args)}' if args else test_cmd
+            console.info(f'Running tests with: [note]{test_cmd}[/note]')
+            subprocess.run(test_cmd, shell=True)
         else:
-            pytest.main(args=[])
+            pytest.main(args=args)
 
     @classmethod
     def from_config(cls) -> Exercise:
@@ -147,30 +159,6 @@ Please [i]install dependencies[/i] (usually inside a virtualenv) and run: [note]
             console.check()
             return True
 
-    @staticmethod
-    def pytest_help():
-        console.info('[note]pypas test[/note] is just a wrapper for [note]pytest[/note]')
-        console.info(
-            'You can find documentation on how to invoke pytest at: [warning]https://docs.pytest.org/en/latest/how-to/usage.html'
-        )
-        console.debug('Here you have some of the more useful options:')
-
-        table = CustomTable(('Command', 'quote'), 'Description')
-        table.add_row('pytest -x', 'Exit instantly on first error or failed test.')
-        table.add_row(
-            'pytest -k <expression>',
-            'Only run tests which match the given substring expression.',
-        )
-        table.add_row('pytest -v', 'Increase verbosity.')
-        table.add_row('pytest -q', 'Decrease verbosity.')
-        table.add_row('pytest --lf', 'Rerun only the tests that failed at the last run.')
-        table.add_row(
-            'pytest --sw',
-            'Exit on test failure and continue from last failing test next time.',
-        )
-        table.add_row('pytest <path/to>::<test>', 'Run specific test.')
-        console.print(table)
-
     @staticmethod
     def log(token: str, frame_ref: str, verbose: bool = False) -> None:
         url = settings.PYPAS_LOG_URLPATH
