@@ -1,10 +1,11 @@
 from pathlib import Path
+from typing import List
 
 import typer
 from rich.prompt import Confirm
 
 from pypas import Config, Exercise, User, console, sysutils
-from pypas.lib.decorators import auth_required, inside_exercise
+from pypas.lib.decorators import auth_required, check_version, inside_exercise
 
 app = typer.Typer(
     add_completion=False,
@@ -25,6 +26,7 @@ def default(
     ),
 ):
     if version:
+        sysutils.handle_package_version()
         print(sysutils.get_package_info())
     # https://typer.tiangolo.com/tutorial/commands/context/#exclusive-executable-callback
     elif ctx.invoked_subcommand is None:
@@ -32,6 +34,7 @@ def default(
 
 
 @app.command()
+@check_version
 def get(exercise_slug: str = typer.Argument(help='Slug of exercise')):
     """Get (download) exercise."""
     if (exercise := Exercise(exercise_slug)).folder_exists():
@@ -41,13 +44,14 @@ def get(exercise_slug: str = typer.Argument(help='Slug of exercise')):
         )
         if not Confirm.ask('Continue', default=False):
             return
-    if (config := Config()).exists():
+    if Config.local_config_exists():
         console.warning('Current folder seems to be a pypas exercise.')
         console.info(
             '[italic]If continue, files coming from server will [red]MESS[/red] your existing files'
         )
         if not Confirm.ask('Continue', default=False):
             return
+    config = Config()
     if exercise.download(config.get('token')):
         exercise.unzip()
         console.info(f'Exercise is available at [note]./{exercise.folder}[/note] [success]✔')
@@ -55,6 +59,7 @@ def get(exercise_slug: str = typer.Argument(help='Slug of exercise')):
 
 @app.command()
 @inside_exercise
+@check_version
 def doc():
     """Open documentation for exercise."""
     exercise = Exercise.from_config()
@@ -63,12 +68,14 @@ def doc():
 
 @app.command()
 @inside_exercise
+@check_version
 def update(
     force: bool = typer.Option(
         False, '--force', '-f', help='Force update and omit backup of existing files'
     ),
 ):
     """Update exercise."""
+    # Backup patterns follow fnmatch syntax: https://docs.python.org/3/library/fnmatch.html
     config = Config()
     if (exercise := Exercise.from_config()).download(config.get('token')):
         dir = exercise.unzip(to_tmp_dir=True)
@@ -76,6 +83,7 @@ def update(
 
 
 @app.command()
+@check_version
 def auth(token: str = typer.Argument(help='Access token')):
     """Authenticate at pypas.es (token required)."""
     if User(token).authenticate():
@@ -84,6 +92,7 @@ def auth(token: str = typer.Argument(help='Access token')):
 
 
 @app.command()
+@check_version
 def upgrade():
     """Upgrade pypas-cli from PyPI."""
     if sysutils.upgrade_pypas():
@@ -94,8 +103,10 @@ def upgrade():
 
 @app.command()
 @inside_exercise
+@check_version
 def zip(verbose: bool = typer.Option(False, '--verbose', '-v', help='Increase verbosity.')):
     """Compress exercise contents."""
+    # Excluded patterns follow fnmatch syntax: https://docs.python.org/3/library/fnmatch.html
     exercise = Exercise.from_config()
     zipfile = exercise.zip(verbose=verbose)
     size, str_size = sysutils.get_file_size(zipfile)
@@ -105,6 +116,7 @@ def zip(verbose: bool = typer.Option(False, '--verbose', '-v', help='Increase ve
 @app.command()
 @auth_required
 @inside_exercise
+@check_version
 def put():
     """Put (upload) exercise."""
     config = Config()
@@ -120,20 +132,19 @@ def put():
     exercise.upload(zipfile, config['token'])
 
 
-@app.command()
+@app.command(context_settings={'ignore_unknown_options': True})
 @inside_exercise
+@check_version
 def test(
-    help: bool = typer.Option(False, '--help', '-h', help='Show test options.'),
+    args: List[str] = typer.Argument(None, help='Arguments passed to test tool'),
 ):
     """Test exercise."""
     exercise = Exercise.from_config()
-    if help:
-        exercise.pytest_help()
-    else:
-        exercise.test()
+    exercise.test(args or [])
 
 
 @app.command()
+@check_version
 def log(
     frame: str = typer.Option('', '--frame', '-f', help='Filter by frame.'),
     verbose: bool = typer.Option(False, '--verbose', '-v', help='Increase verbosity.'),
@@ -144,6 +155,7 @@ def log(
 
 
 @app.command()
+@check_version
 def list(
     frame: str = typer.Option('', '--frame', '-f', help='Filter by frame.'),
     primary_topic: str = typer.Option('', '--ptopic', '-p', help='Filter by primary topic.'),
@@ -155,6 +167,7 @@ def list(
 
 
 @app.command()
+@check_version
 def unauth():
     """Unauthenticate from pypas.es (clear token)."""
     config = Config()
@@ -163,6 +176,7 @@ def unauth():
 
 
 @app.command()
+@check_version
 def run():
     """Run exercise with given args."""
     exercise = Exercise.from_config()
@@ -170,6 +184,7 @@ def run():
 
 
 @app.command()
+@check_version
 def pull(
     item_slug: str = typer.Argument(help='Slug of exercise or frame.'),
 ):
