@@ -14,9 +14,7 @@ PYPAS_PULL_URLPATH = urljoin(PYPAS_BASE_URL, '/assignments/pull/{item_slug}/')
 EXERCISE_CONFIG_FILE = config('EXERCISE_CONFIG_FILE', default='.pypas.toml')
 MAIN_CONFIG_FILE = config('MAIN_CONFIG_FILE', default=Path.home() / '.pypas.toml', cast=Path)
 LARGE_FILE_SIZE = config('LARGE_FILE_SIZE', default=1024 * 1024, cast=int)
-ZIP_IGNORED_PATTERNS = config(
-    'ZIP_IGNORED_PATTERNS',
-    default=r'cache,node_modules,DS_Store,venv,\.git,\.idea',
-    cast=config.list,
+
+PYPAS_SKIP_VERSION_CHECK_VAR = config(
+    'PYPAS_SKIP_VERSION_CHECK_VAR', default='PYPAS_SKIP_VERSION_CHECK'
 )
-ZIP_IGNORED_PATTERNS_RE = '|'.join(ZIP_IGNORED_PATTERNS)
